@@ -8,7 +8,7 @@
 use arrayvec::ArrayVec;
 use hir::{
     Adt, AsAssocItem, AssocItem, BuiltinAttr, BuiltinType, Const, Field, Function, GenericParam,
-    HasVisibility, Impl, ItemInNs, Label, Local, MacroDef, Module, ModuleDef, Name, PathResolution,
+    HasVisibility, Impl, ItemInNs, Label, Local, Macro, Module, ModuleDef, Name, PathResolution,
     Semantics, Static, ToolModule, Trait, TypeAlias, Variant, Visibility,
 };
 use stdx::impl_from;
@@ -22,7 +22,7 @@ use crate::RootDatabase;
 // FIXME: a more precise name would probably be `Symbol`?
 #[derive(Debug, PartialEq, Eq, Copy, Clone, Hash)]
 pub enum Definition {
-    Macro(MacroDef),
+    Macro(Macro),
     Field(Field),
     Module(Module),
     Function(Function),
@@ -48,7 +48,7 @@ impl Definition {
 
     pub fn module(&self, db: &RootDatabase) -> Option<Module> {
         let module = match self {
-            Definition::Macro(it) => it.module(db)?,
+            Definition::Macro(it) => it.module(db),
             Definition::Module(it) => it.parent(db)?,
             Definition::Field(it) => it.parent_def(db).module(db),
             Definition::Function(it) => it.module(db),
@@ -94,7 +94,7 @@ impl Definition {
 
     pub fn name(&self, db: &RootDatabase) -> Option<Name> {
         let name = match self {
-            Definition::Macro(it) => it.name(db)?,
+            Definition::Macro(it) => it.name(db),
             Definition::Field(it) => it.name(db),
             Definition::Module(it) => it.name(db)?,
             Definition::Function(it) => it.name(db),
@@ -116,6 +116,7 @@ impl Definition {
     }
 }
 
+#[derive(Debug)]
 pub enum IdentClass {
     NameClass(NameClass),
     NameRefClass(NameRefClass),
@@ -145,6 +146,15 @@ impl IdentClass {
         Self::classify_node(sema, &parent)
     }
 
+    pub fn classify_lifetime(
+        sema: &Semantics<RootDatabase>,
+        lifetime: &ast::Lifetime,
+    ) -> Option<IdentClass> {
+        NameRefClass::classify_lifetime(sema, lifetime)
+            .map(IdentClass::NameRefClass)
+            .or_else(|| NameClass::classify_lifetime(sema, lifetime).map(IdentClass::NameClass))
+    }
+
     pub fn definitions(self) -> ArrayVec<Definition, 2> {
         let mut res = ArrayVec::new();
         match self {
@@ -296,7 +306,7 @@ impl NameClass {
 
         if let Some(it) = ast::LifetimeParam::cast(parent.clone()) {
             sema.to_def(&it).map(Into::into).map(Definition::GenericParam)
-        } else if let Some(it) = ast::Label::cast(parent.clone()) {
+        } else if let Some(it) = ast::Label::cast(parent) {
             sema.to_def(&it).map(Definition::Label)
         } else {
             None
@@ -437,7 +447,7 @@ impl NameRefClass {
 
 impl_from!(
     Field, Module, Function, Adt, Variant, Const, Static, Trait, TypeAlias, BuiltinType, Local,
-    GenericParam, Label
+    GenericParam, Label, Macro
     for Definition
 );
 
@@ -482,9 +492,8 @@ impl From<PathResolution> for Definition {
             }
             PathResolution::Local(local) => Definition::Local(local),
             PathResolution::TypeParam(par) => Definition::GenericParam(par.into()),
-            PathResolution::Macro(def) => Definition::Macro(def),
-            PathResolution::SelfType(impl_def) => Definition::SelfType(impl_def),
             PathResolution::ConstParam(par) => Definition::GenericParam(par.into()),
+            PathResolution::SelfType(impl_def) => Definition::SelfType(impl_def),
             PathResolution::BuiltinAttr(attr) => Definition::BuiltinAttr(attr),
             PathResolution::ToolModule(tool) => Definition::ToolModule(tool),
         }
@@ -502,6 +511,7 @@ impl From<ModuleDef> for Definition {
             ModuleDef::Static(it) => Definition::Static(it),
             ModuleDef::Trait(it) => Definition::Trait(it),
             ModuleDef::TypeAlias(it) => Definition::TypeAlias(it),
+            ModuleDef::Macro(it) => Definition::Macro(it),
             ModuleDef::BuiltinType(it) => Definition::BuiltinType(it),
         }
     }
